#!/usr/bin/python3
import requests
import argparse
import datetime
import os
import json
from html.parser import HTMLParser

STD_URL = "https://www.gruppenhaus.de/ajax_belplan.php"

WEEKDAY_MAPPING = ['So', 'Mo', 'Di', 'Mi', 'Do', 'Fr', 'Sa', 'So']

class BookingScheduleParser(HTMLParser):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.schedule = {'schedule': []}
        self._parseMonth = None
        self._dayState = None
        self._regDayFound = True
        self._monthIdx = -1

    def handle_starttag(self, tag, attrs):
        for attr in attrs:
            if tag == 'td' and attr[0] == 'class':
                classList = attr[1].split(' ')
                if len(classList) > 0 and classList[0] in ['superweiss', 'blgt', 'frei', 'anft', 'abft']:
                    self._regDayFound = True
                    self._dayState = classList[0]

    def handle_endtag(self, tag):
        self._regDayFound = False

    def handle_data(self, data):
        if self._regDayFound and self._parseMonth and self._dayState:
            fullDateStr = self.schedule['schedule'][self._monthIdx]['date'] + '-{:0>2d}'.format(int(data))
            fullDate = datetime.datetime.strptime(fullDateStr, '%Y-%m-%d')
            weekDay = int(fullDate.strftime('%w'))
            week = int(fullDate.strftime('%W'))
            weekDayName = WEEKDAY_MAPPING[weekDay]
            if fullDate.year >= 2025 and self._dayState == 'frei':
                self._dayState = 'superweiss'

            self.schedule['schedule'][self._monthIdx]['items'].append({
                'day': int(data),
                'date': fullDateStr,
                'week': week,
                'weekday': weekDay,
                'weekdayName': weekDayName,
                'state': self._dayState
            })

    def feed(self, year, month, *args, **kwargs):
        self._parseMonth = '{:0>4d}-{:0>2d}'.format(year,month)
        self._monthIdx += 1
        self.schedule['schedule'].append({
            'date': self._parseMonth,
            'items': []
        })
        super().feed(*args, **kwargs)

def updateYear(args) -> None:
    parser = BookingScheduleParser()

    now = datetime.datetime.now()
    for i in range(0, args.duration):
        calValue = 'cal_1_hs_{}_cmy_{month:0>2d}{year:0>4d}'.format(
            args.hausId,
            month=now.month,
            year=now.year
        )
        data = {
            'kalender': calValue
        }
        headers = {
            'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64; rv:100.0) Gecko/20100101 Firefox/100.0',
            'X-Requested-With': 'XMLHttpRequest'
        }
        resp = requests.post(args.apiUrl, data=data, headers=headers)
        if resp.ok:
            parser.feed(now.year, now.month, resp.text)
        
        # switch to next month
        month = now.month + 1
        year = now.year
        if month > 12:
            month = 1
            year += 1
        now = now.replace(month=month, year=year)
    
    baseDir = os.path.dirname(args.output)
    if baseDir:
        os.makedirs(baseDir, exist_ok=True)
    with open(args.output, 'w', encoding='utf-8') as f:
        f.write(json.dumps(parser.schedule))

def cli():
    parser = argparse.ArgumentParser()
    parser.add_argument('--haus', dest='hausId', default=4602, type=int, help='Gruppenhaus ID')
    parser.add_argument('--url', '-u', dest='apiUrl', default=STD_URL, help='Gruppenhaus API URL')
    parser.add_argument('--duration', dest='duration', type=int, default=12, help='Anz. an Monaten')
    parser.add_argument('--output', '-o', dest='output', required=True, help='Ausgabedatei (json)')
    return parser.parse_args()

if __name__ == '__main__':
    args = cli()
    updateYear(args)
