// Parts extracted from Bootstrap 5!
const CLASS_NAME_START = 'carousel-item-start';
const CLASS_NAME_END = 'carousel-item-end';
const CLASS_NAME_NEXT = 'carousel-item-next';
const CLASS_NAME_PREV = 'carousel-item-prev';
const CLASS_NAME_ACTIVE = 'active';
const EVENT_SLIDE = 'slide.bs.carousel';
const EVENT_SLID = 'slid.bs.carousel';
const ORDER_NEXT = 'next';
const ORDER_PREV = 'prev';
const DIRECTION_LEFT = 'left';
const DIRECTION_RIGHT = 'right';
const TRANSITION_END = 'transitionend';
const MILLISECONDS_MULTIPLIER = 1000;

class Carousel {
    constructor(container) {
        console.log("Initializing Carousel");
        this._container = container;
        this._indicatorsElement = container.querySelector('.carousel-indicators');
        this._interval = null;
        this._isSliding = false;
        this._activeElement = null;
        this._addEventListener();
        this.cycle();
    }

    _queueCallback(callback, element, isAnimated = true) {
        executeAfterTransition(callback, element, isAnimated);
    }

    _getTimeout() {
        var timeout = this._container.getAttribute('data-slide-interval');
        if (timeout) {
            timeout = parseInt(timeout);
        }
        if (!timeout || timeout === NaN || timeout <= 0) {
            timeout = 5000;
        }
        return timeout;
    }

    _installTimer() {
        var car = this;
        this._interval = setInterval(function () {
            car.next();
        }, this._getTimeout());
    }

    cycle() {
        this._installTimer();
    }

    pause() {
        if (this._interval) {
            clearInterval(this._interval);
            this._interval = null;
        }
    }

    _addEventListener() {
        if (this._indicatorsElement) {
            const indic = Array.from(this._indicatorsElement.querySelectorAll('[data-bs-slide-to]'));
            indic.forEach((elem) => {
                elem.addEventListener('click', (event) => {
                    const slideTo = Number.parseInt(event.target.getAttribute('data-bs-slide-to'));
                    if (slideTo != NaN) {
                        this.goto(slideTo);
                    }
                })
            })
        }
    }

    _getItemIndex(element) {
        this._items = element && element.parentNode ? Array.from(element.parentNode.querySelectorAll('.carousel-item')) : [];
        return this._items.indexOf(element);
    }

    _setActiveIndicatorElement(element) {
        if (this._indicatorsElement) {
            const activeIndicator = this._indicatorsElement.querySelector('.active');
            activeIndicator.classList.remove('active');
            activeIndicator.removeAttribute('aria-current');
            const indicators = this._indicatorsElement.querySelectorAll('[data-bs-target]');

            for (let i = 0; i < indicators.length; i++) {
                if (Number.parseInt(indicators[i].getAttribute('data-bs-slide-to'), 10) === this._getItemIndex(element)) {
                    indicators[i].classList.add('active');
                    indicators[i].setAttribute('aria-current', 'true');
                    break;
                }
            }
        }
    }

    next() {
        this._slide(ORDER_NEXT);
    }

    prev() {
        this._slide(ORDER_PREV);
    }

    goto(nextIdx) {
        if (!this._activeElement) {
            this._activeElement = this._container.querySelector('.carousel-item.active');
        }
        const activeIndex = this._getItemIndex(this._activeElement);
        if (nextIdx > this._items.length - 1 || nextIdx < 0) {
            return;
        }

        // Same index is requested. Just reset timer.
        if (activeIndex === nextIdx) {
            this.pause();
            this.cycle();
            return;
        }

        const order = nextIdx > activeIndex ? ORDER_NEXT : ORDER_PREV;
        this._slide(order, this._items[nextIdx]);
    }

    _directionToOrder(direction) {
        if (![DIRECTION_RIGHT, DIRECTION_LEFT].includes(direction)) {
            return direction;
        }
        return direction === DIRECTION_LEFT ? ORDER_NEXT : ORDER_PREV;
    }

    _getNextElement() {
        var idx = this._items.indexOf(this._activeElement);
        var nextIdx = idx + 1;
        if (nextIdx >= this._items.length) {
            nextIdx = 0;
        }
        return this._items[nextIdx];
    }

    _slide(directionOrOrder, element=undefined) {
        const order = this._directionToOrder(directionOrOrder);
        const isNext = order === ORDER_NEXT;
        const directionalClassName = isNext ? CLASS_NAME_START : CLASS_NAME_END;
        const orderClassName = isNext ? CLASS_NAME_NEXT : CLASS_NAME_PREV;
        const isCycling = Boolean(this._interval);

        const activeElement = this._container.querySelector('.carousel-item.active');
        if (!this._activeElement) {
            this._activeElement = activeElement;
        }
        const activeIndex = this._getItemIndex(activeElement);
        if (this._items.length <= 1) {
            console.warn("Skip iterating carousel. There is only one item.");
            return;
        }
        const nextElement = element || this._getNextElement();

        if (isCycling) {
            this.pause();
        }
        // Update indicators... the next one should already be set.
        this._setActiveIndicatorElement(nextElement);
        // Continue
        this._activeElement = nextElement;
        // Sliding?
        if (this._container.classList.contains("slide")) {
            nextElement.classList.add(orderClassName);
            reflow(nextElement);
            activeElement.classList.add(directionalClassName);
            nextElement.classList.add(directionalClassName);
            const completeCallBack = () => {
                nextElement.classList.remove(directionalClassName, orderClassName);
                nextElement.classList.add(CLASS_NAME_ACTIVE);
                activeElement.classList.remove(CLASS_NAME_ACTIVE, orderClassName, directionalClassName);
                this._isSliding = false;
            };
            this._queueCallback(completeCallBack, activeElement, true);
        } else {
            activeElement.classList.remove(CLASS_NAME_ACTIVE);
            nextElement.classList.add(CLASS_NAME_ACTIVE);
            this._isSliding = false;
        }

        if (isCycling) {
            this.cycle();
        }
    }
}

const reflow = element => element.offsetHeight;
const execute = callback => {
    if (typeof callback === 'function') {
        callback();
    }
};
const getTransitionDurationFromElement = element => {
    if (!element) {
        return 0;
    } // Get transition-duration of the element
    let {
        transitionDuration,
        transitionDelay
    } = window.getComputedStyle(element);
    const floatTransitionDuration = Number.parseFloat(transitionDuration);
    const floatTransitionDelay = Number.parseFloat(transitionDelay); // Return 0 if element or transition duration is not found

    if (!floatTransitionDuration && !floatTransitionDelay) {
        return 0;
    } // If multiple durations are defined, take the first
    transitionDuration = transitionDuration.split(',')[0];
    transitionDelay = transitionDelay.split(',')[0];
    return (Number.parseFloat(transitionDuration) + Number.parseFloat(transitionDelay)) * MILLISECONDS_MULTIPLIER;
};
const triggerTransitionEnd = element => {
    element.dispatchEvent(new Event(TRANSITION_END));
};
const executeAfterTransition = (callback, transitionElement, waitForTransition = true) => {
    if (!waitForTransition) {
        execute(callback);
        return;
    }

    const durationPadding = 5;
    const emulatedDuration = getTransitionDurationFromElement(transitionElement) + durationPadding;
    let called = false;

    const handler = ({
        target
    }) => {
        if (target !== transitionElement) {
            return;
        }

        called = true;
        transitionElement.removeEventListener(TRANSITION_END, handler);
        execute(callback);
    };

    transitionElement.addEventListener(TRANSITION_END, handler);
    setTimeout(() => {
        if (!called) {
            triggerTransitionEnd(transitionElement);
        }
    }, emulatedDuration);
};

// Init carousel
var slides = document.getElementsByClassName('carousel');
for (var i = 0; i < slides.length; i++) {
    new Carousel(slides.item(i));
}
